<?php
/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 21/01/14
 * Time: 14:13
 * To change this template use File | Settings | File Templates.
 */

namespace Fiscal\Logic\NotaFiscalEletronica\TransmissaoNfEletronica;

use Fiscal\Entities\Notafiscal;
use Fiscal\Logic\NotaFiscalEletronica\ToolsNFePHP;
//use Fiscal\Logic\NotaFiscalEletronica\V200;
use Fiscal\Logic\NotaFiscalEletronica\V300;
use Geral\Entities\Empresa;
use Geral\Entities\Lotenfe;
use Geral\Entities\Paramfilial;
use Geral\Entities\Tipooperac;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use Geral\Util\Util;
use Indeterminado\Entities\Arqxmlnfe;
use Indeterminado\Entities\Relacnflotenfe;
use Setforms2\Shared\Registry;


class TransmissaoNfe
{

    const ID_ENTRADA = 'E';
    const ID_SAIDA = 'S';
    const AMBIENTE_PRODUCAO = '1';
    const AMBIENTE_HOMOLOGACAO = '2';
    const STATUS_NF_TRANSMITIDA = 'T';
    const BLOQUEADO = 'S';
    const DESBLOQUADO = 'N';

    const EVENTO_CANCELAMENTO_REG = '110111';

    const PREFIXO_CONTADOR_NOTAFISCAL = 'LANCNF/FILIAL';

    const PREFIXO_CONTADOR_NOTAFISCALELE = 'NOTAFISCALELE';
    const TAMANHO_NOTAFISCALELE = 9;

    private $notasfiscais = array();

    /** @var Empresa */
    private $empresa;
    private $nrorg;
    private $identrsaid;
    private $ambiente;
    private $modulo;
    private $nrlote;
    private $modoserie;
    /** @var \Fiscal\Logic\NotaFiscalEletronica\V200 */
    //    private $v200;
    /** @var \Fiscal\Logic\NotaFiscalEletronica\V300 */
    private $v300;
    /** @var \Fiscal\Logic\NotaFiscalEletronica\ToolsNFePHP */
    private $toolnfe;
    private $arrayErros = array();

    /**
     * @param        $cdempresa
     * @param array  $notasfiscais
     * @param        $identrsaid ' E' = ENTRADA
     *                           'S' = SAÍDA
     * @param string $ambiente
     * @param        $nrorg      Organização
     * @param int    $modoSerie
     * @param        $modulo
     * @param        $nrlote
     * @throws \Exception
     * @internal param $ array|\Fiscal\Entities\Notafiscal|\Fiscal\Repositories\NotaFiscal
     */
    function __construct($empresa, $notasfiscais = Array(), $identrsaid, $ambiente = self::AMBIENTE_HOMOLOGACAO, $nrorg,
                         $modoSerie = Notafiscal::MODO_SERIE_NORMAL, $modulo, $nrlote)
    {

        try {
            $this->notasfiscais = $notasfiscais;
            $this->identrsaid   = $identrsaid;
            $this->ambiente     = $ambiente;
            $this->nrorg        = $nrorg;
            $this->modoserie    = $modoSerie;
            $this->modulo       = $modulo;
            $this->nrlote       = $nrlote;
            $this->empresa      = $empresa;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function verificaStatusServicos($cdempresa, $nrorg, $cdinscesta)
    {
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
        //Instancia da Classe ToolsNfe
        $toolsnfe = new ToolsNFePHP($empresa, $cdinscesta);
        if ($toolsnfe->errStatus) {
            throw new \Exception($toolsnfe->errMsg);
        }

        //Verifica status do serviço SVAN
        $svan = $toolsnfe->statusServico($toolsnfe->config['UF'], $toolsnfe->config['ambiente'], '2', true);
        if (is_array($svan)) {
            $toolsnfe->enableSVAN = $svan['bStat'];
        }
        if (!$toolsnfe->enableSVAN) {
            //Verifica status do serviço da SEFAZ
            $sefaz = $toolsnfe->statusServico($toolsnfe->config['UF'], $toolsnfe->config['ambiente']);
            if (is_array($sefaz)) {
                $toolsnfe->statusSEFAZ = $sefaz['cStat'];
            }
        }

        return $toolsnfe;
    }

    /**
     * Este Método gera uma nova Nota Fiscal com os mesmos dados da Nota Original mais os dados do Cancelamento,
     * a Nota Original será alterada para pendente.
     * @param Notafiscal $notafiscalorigem
     * @param            $xmlCancelamento string
     * @throws \Exception
     * @internal param $justificativa
     */
    public static function cancelaNfe(NotaFiscal $notafiscalorigem, $xmlCancelamento)
    {

        try {
            $doc                    = new \DOMDocument();
            $doc->preservWhiteSpace = false; //elimina espaços em branco
            $doc->formatOutput      = false;
            $doc->loadXML($xmlCancelamento);

            //Status
            $cstat = $doc->getElementsByTagName('cStat')->item(0)->nodeValue;

            //Justificativa
            $justificativa = $doc->getElementsByTagName('xJust')->item(0)->nodeValue;

            //Motivo
            $motivo = $doc->getElementsByTagName('xMotivo')->item(0)->nodeValue;

            //Data e Hora do Protocolo
            $dthoraprot = $doc->getElementsByTagName('dhRegEvento')->item(0)->nodeValue;
            self::formatDtHrProt($dthoraprot);

            // Tipo do Evento
            $tpEvento = $doc->getElementsByTagName('tpEvento')->item(0)->nodeValue;
            if ($tpEvento == self::EVENTO_CANCELAMENTO_REG) {
                $motivo .= "(Dados da NF Cancelada: Filial: {$notafiscalorigem->getCdfilial()}, Lançamento: {$notafiscalorigem->getNrlanctonf()})";
            }

            //Insere a nova Nota Fiscal
            /** @var $novaNotafiscal NotaFiscal */
            $novaNotafiscal = clone $notafiscalorigem;
            $novaNotafiscal->setNrlanctonf(
                NovoCodigo::geraCodigo(
                    $notafiscalorigem->getNrorg(),
                    self::PREFIXO_CONTADOR_NOTAFISCAL .
                    $notafiscalorigem->getCdfilial()
                )
            );
            $novaNotafiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_BLOQUEADA);
            $novaNotafiscal->setIdstatus(Notafiscal::CANCELADO);
            $novaNotafiscal->setDtcancelanf($dthoraprot);
            $novaNotafiscal->setCdopecancnf(Ambiente::getCdOperador());
            $novaNotafiscal->setIdstatusnfe(Notafiscal::CANCELADO);
            $novaNotafiscal->setDsmotivocancnfe($justificativa);
            $novaNotafiscal->setDsobsstatusnfe($motivo);
            $novaNotafiscal->persist();
            Registry::getEntityManager()->flush();

            //Updata a Nota Fiscal de Origem
            $notafiscalorigem->setNracessonfe(null);
            $notafiscalorigem->setIdstatusnfe(null);
            $notafiscalorigem->setNrprotocolonfe(null);
            $notafiscalorigem->setIdstatusnfe(Notafiscal::PENDENTE);
            $notafiscalorigem->setNrnotafiscalele(null);
            $notafiscalorigem->setDsobsstatusnfe(null);
            $notafiscalorigem->setCdemplotenfe(null);
            $notafiscalorigem->setNrlotenfe(null);
            $notafiscalorigem->setIdbloqueado(Notafiscal::NOTA_FISCAL_NAO_BLOQUEADA);
            $notafiscalorigem->setIdstatus(Notafiscal::PENDENTE);
            $notafiscalorigem->setDtsaidemis(null);
            $notafiscalorigem->setDtemissao(null);
            $notafiscalorigem->setCdoperapro(null);
            $notafiscalorigem->setNrlanctbdoc(null);
            $notafiscalorigem->setDtexprnf(null);
            $notafiscalorigem->setNrexprnf(null);
            $notafiscalorigem->setIdtipoemisnfe(NotaFiscal::IDTIPOEMISNFE_NAO_EMITIDO);
            $notafiscalorigem->setIdemailnfe('P');
            $notafiscalorigem->setIdemailnfetransp('P');
            $notafiscalorigem->setIdtpambnfe(Notafiscal::IDTPAMBNFE_0);
            $notafiscalorigem->setCdbarraconting(null);
            $notafiscalorigem->setDthrprotocolnfe(null);
            $notafiscalorigem->setCdparamemisre(null);
            $notafiscalorigem->setNrseqcapadoc(null);
            $notafiscalorigem->setDthrprotocolnfe(null);
            $notafiscalorigem->setNrregdpecnfe(null);
            $notafiscalorigem->setNrregdpecnfe(null);
            $notafiscalorigem->persist();
            Registry::getEntityManager()->flush();

            //Seta os dados do calcelamento na tab
            tk_reload_data_with_filter('tab_pbi00157_000', 'win_pbi00157_000');
            tk_set_field_value('IDSTATUSNFE', $cstat);
            tk_set_field_value('DSOBSSTATUSNFE', $motivo);
            tk_set_field_value('NRPROTOCOLONFE', $notafiscalorigem->getNrprotocolonfe());
            tk_set_field_value('DTHRPROTOCOLNFE', $dthoraprot->format('d/m/Y H:i:s'));
            tk_disable_field('BTCONFIRMA');
        } catch (\Exception $e) {
            throw $e;
        }

    }

    private function formatDtHrProt(&$dthoraprot)
    {
        $dthoraprot = substr($dthoraprot, 8, 2) . '/' . substr($dthoraprot, 5, 2) . '/' . substr($dthoraprot, 0, 4) . ' ' . substr($dthoraprot, 11, 8);
        $dthoraprot = DateUtil::getDataDeString($dthoraprot, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
    }

    public function getArrayErros()
    {
        return $this->arrayErros;
    }

    public function getToolnfe()
    {
        return $this->toolnfe;
    }

    public function getIdentrsaid()
    {
        return $this->identrsaid;
    }

    public function getNotafiscal()
    {
        return $this->notafiscal;
    }

    /**
     * @return bool
     * @throws \Exception
     */
    public function transmite($metodoTransmissao, $cdinscesta)
    {
        try {
            //           //Instancia da Classe ToolsNfe
            $this->toolnfe = new ToolsNFePHP(self::getEmpresa(), $cdinscesta);
            if ($this->toolnfe->errStatus) {
                throw new \Exception($this->toolnfe->errMsg);
            }

            //Utilizado para verificar se houve erro durante o envio das Nfes
            $envio = false;

            // Array com os nomes dos Xmls das Notas Fiscais Assinadas e Validadas com os XSd
            $nfenameFiles = array();

            /** @var $notafiscal NotaFiscal */
            foreach ($this->notasfiscais as $notafiscal) {
                //Gera arquivos XMl da Nota Fiscal e grava na pasta Entradas
                self::geraXML($notafiscal);
                self::gravaNumeroNotafiscalEle($notafiscal);

                //Assinatura
                if (self::assinaNotaFiscal()) {
                    //Validação com XSD
                    if (self::validaXmlXsd()) {
                        //                         Aguardado resolução do problema do long raw SOL. 0000143520
                        //                        Gravação do arquivo no Banco de Dados
                        /*$nfefile = file_get_contents($this->toolnfe->valDir.$this->v300->getfilename());
                        if($nfefile){
                            self::gravaXml($notafiscal->getNrorg(),
                                           $nfefile,
                                           $notafiscal->getCdfilial(),
                                           $notafiscal->getNrlanctonf(),
                                           Arqxmlnfe::TIPO_ARQ_XML_DADOS_NOTA_FISCAL);

                        }else{
                            throw new \Exception("Arquivo validado não encontrado.");
                        }*/
                        // Armazena no array os nomes dos arquivos XML
                        $nfenameFiles[] = $this->v300->getfilename();
                    }
                }
            }
            if (count($nfenameFiles) > 0) {
                // Enviar Notas p/ SEFAZ
                $envio = $this->toolnfe->EnvNFe($this->getNrlote(), $nfenameFiles, $metodoTransmissao);
                if ($metodoTransmissao == 'SVAN') {
                    $this->modoserie = $this->toolnfe->modoSerie;
                }
                if ($envio) {
                    self::insereLoteNfe();
                    self::updateNotasFiscais();
                }
            }
            return $envio;
        } catch (\Exception $e) {
            throw ($e);
        }
    }

    /**
     * @return \Geral\Entities\Empresa
     */
    public function getEmpresa()
    {
        return $this->empresa;
    }

    public function setEmpresa($cdempresa, $nrorg)
    {
        $this->empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
    }

    /**
     * Gera os arquivos xmls das Notas Fiscais na Pasta Entradas
     *
     */
    private function geraXML(NotaFiscal $notafiscal)
    {
        try {
            // Seta os dados da nota fiscal
            $corpoNf    = self::getCorpoNf($notafiscal);
            //            $this->v200 = new V200(
            //                $corpoNf,
            //                $this->getAmbiente(),
            //                $this->getModulo(),
            //                $this->getModulo(),
            //                $this->getNrlote()
            //            );

            $this->v300 = new V300(
                $corpoNf,
                $this->getAmbiente(),
                $this->getModoserie(),
                $this->getModulo(),
                $this->getNrlote()
            );

            self::insereChaveAcesso($notafiscal);
            //            $this->v200->geraArquivoXML();
            $this->v300->geraArquivoXML();

        } catch (\Exception $e) {
            throw $e;
        }

    }

    private function getCorpoNf(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        /* @@var \Fiscal\Repositories\Notafiscal */
        return Notafiscal::getRepository()->getCorpoNotaFiscal(
            $notafiscal->getCdfiliemit(),
            $notafiscal->getNrlanctonf(),
            $notafiscal->getIdorigdestnf(),
            $notafiscal->getIdentrsaidop(),
            $notafiscal->getNrorg()
        );
    }

    public function getAmbiente()
    {
        return $this->ambiente;
    }

    public function getModulo()
    {
        return $this->modulo;
    }

    public function getNrlote()
    {
        return $this->nrlote;
    }

    /**
     * @param $notafiscal \Fiscal\Entities\NotaFiscal
     */
    private function insereChaveAcesso($notafiscal)
    {
        //Updata Chave de Acesso da Nf
        //        $notafiscal->setNracessonfe($this->v200->getChaveAcesso());
        $notafiscal->setNracessonfe($this->v300->getChaveAcesso());
        $notafiscal->persist();
        Registry::getEntityManager()->flush();
    }

    /**
     * @return bool|mixed
     * @throws \Exception
     */
    private function assinaNotaFiscal()
    {
        try {

            //            $file = $this->toolnfe->entDir . $this->v200->getfilename();
            $file = $this->toolnfe->entDir . $this->v300->getfilename();
            if (is_file($file)) {
                $nfefile = file_get_contents($file);
                if ($signn = $this->toolnfe->signXML($nfefile, 'infNFe')) {
                    //xml retornado gravar
                    //                    $file = $this->toolnfe->assDir . $this->v200->getfilename();
                    $file = $this->toolnfe->assDir . $this->v300->getfilename();
                    if (!file_put_contents($file, $signn)) {
                        $this->errStatus = true;
                        return false;
                    } else {
                        //Remove da pasta Entradas
                        //                        unlink($this->toolnfe->entDir . $this->v200->getfilename());
                        unlink($this->toolnfe->entDir . $this->v300->getfilename());
                    } //fim do teste de gravação
                } //fim do teste da assinatura
                else {
                    throw new \Exception($this->toolnfe->errMsg);
                }
            } else {
                throw new \Exception('Erro: Arquivo xml de entrada não encontrado.');
            }
            return $signn;


        } catch (\Exception $e) {
            throw $e;
        }

    }

    private function validaXmlXsd()
    {
        try {
            //            $nfefile = $this->toolnfe->assDir . $this->v200->getfilename();
            $nfefile = $this->toolnfe->assDir . $this->v300->getfilename();
            if (is_file($nfefile)) {
                //                $valid = $this->toolnfe->validNFe($this->v200->getfilename());
                $valid = $this->toolnfe->validNFe($this->v300->getfilename());
                if (!$valid) {
                    throw new \Exception($this->toolnfe->errMsg);
                }
                return $valid;
            } else {
                throw new \Exception('Erro: Arquivo xml assinado não encontrado.');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereLoteNfe()
    {
        try {
            $lotenfe = new Lotenfe($this->empresa->getCdempresa(), $this->getNrorg());
            $lotenfe->setNrlotenfe(self::getNrlote());
            $lotenfe->setNrrecibonfe($this->toolnfe->nrrecibo);
            $lotenfe->setIdtpamblotenfe($this->getAmbiente());
            $lotenfe->setIdtipoemislote($this->modoserie);
            $lotenfe->setIdstatuslotenfe(Lotenfe::STATUS_TRANSMITIDO);
            $lotenfe->setDtenviolotenfe(DateUtil::getDataAtual());
            $lotenfe->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getNrorg()
    {
        return $this->nrorg;
    }

    /**
     * Updata os dados das notas fiscais enviadas (Status, Chave de Acesso, Nº do Lote)
     *
     */

    public function updateNotasFiscais()
    {

        try {
            /** @var $notafiscal NotaFiscal */
            foreach ($this->notasfiscais as $notafiscal) {
                $notafiscal->setIdstatusnfe(self::STATUS_NF_TRANSMITIDA);
                $notafiscal->setNrlotenfe(self::getNrlote());
                $notafiscal->setCdemplotenfe($notafiscal->getCdempresa());
                $notafiscal->setIdbloqueado(self::BLOQUEADO);
                $notafiscal->setIdtpambnfe(self::getAmbiente());
                $notafiscal->setIdtipoemisnfe(self::getModoserie());
                //                $notafiscal->setNrnotafiscalele($this->v200->getNumeroNotaFiscal());
                //                $notafiscal->setNrnotafiscalele($this->v300->getNumeroNotaFiscal());
                $dtprocessa = self::getDtProcessaFilial(
                    $notafiscal->getCdfilial(),
                    $notafiscal->getNrorg()
                );
                if ($dtprocessa) {
                    $notafiscal->setDtaprovacao($dtprocessa);
                }


                //                if (!$notafiscal->getNrnotafiscalele()) {
                //                    $nrnotafiscalele = self::PrxNrNotaFiscalEle(
                //                        $notafiscal->getCdempresa(),
                //                        $notafiscal->getNrinsjurnf(),
                //                        $notafiscal->getCdserie(),
                //                        self::getAmbiente(),
                //                        $notafiscal->getNrorg()
                //                    );
                //                } else {
                //                    $nrnotafiscalele = $notafiscal->getNrnotafiscalele();
                //                }

                //                $notafiscal->setNrnotafiscalele($nrnotafiscalele);
                //                $notafiscal->setDtemissao(DateUtil::getDataDeString($this->v200->corpoNF['DTEMISSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA));
                $notafiscal->setDtemissao(DateUtil::getDataDeString($this->v300->corpoNF['DTEMISSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA));
                $notafiscal->setNrnotafisc($notafiscal->getNrnotafiscalele());
                $notafiscal->persist();
				
                self::insereRelacnflotenfe($notafiscal);

                Registry::getEntityManager()->flush();
                self::geraVencimento($notafiscal);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
	public function geraVencimento(Notafiscal $notafiscal){
		if($notafiscal->getIddocfinanc() != 'L'){
			$tipoOperacao = Tipooperac::findOneByCdtipooperAndIdentrsaidopAndNrorg($notafiscal->getCdtipooper(), $notafiscal->getIdentrsaidop(), $notafiscal->getNrorg());
			if($tipoOperacao->getIdutilvenc() != 'N'){
				ProceduresNF::procedureGeraVencimento($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), 'T');
			}
		}
	}

    public function gravaNumeroNotafiscalEle(Notafiscal $notafiscal){
        $notafiscal->setNrnotafiscalele($this->v300->getNumeroNotaFiscal());

        if (!$notafiscal->getNrnotafiscalele()) {
            $notafiscal->setNrnotafiscalele(self::PrxNrNotaFiscalEle(
                    $notafiscal->getCdempresa(),
                    $notafiscal->getNrinsjurnf(),
                    $notafiscal->getCdserie(),
                    self::getAmbiente(),
                    $notafiscal->getNrorg()));
        }

        $notafiscal->persist();
        Registry::getEntityManager()->flush();
    }

    public function getModoserie()
    {
        return $this->modoserie;
    }

    private function getDtProcessaFilial($cdfilial, $nrorg)
    {
        $paramfilial = Paramfilial::findOneByCdfilialAndNrorg($cdfilial, $nrorg);
        if ($paramfilial) {
            return $paramfilial->getDtprocessa();
        } else {
            return null;
        }
    }

    private function PrxNrNotaFiscalEle($cdempresa, $nrinsjuremit, $serienf, $idtpambiente, $nrorg)
    {
        $prxnrnotafiscalele = NovoCodigo::geraCodigo(
            $nrorg,
            self::PREFIXO_CONTADOR_NOTAFISCALELE . $cdempresa .
            trim(Util::removeMascaraInscricao($nrinsjuremit)) . $serienf .
            $idtpambiente,
            1,
            self::TAMANHO_NOTAFISCALELE
        );
        return $prxnrnotafiscalele;
    }

    public function validaNotafiscalTransmissao()
    {
        $validacao        = new ValidacaoNfEletronica($this->notasfiscais, $this->modulo);
        $this->arrayErros = $validacao->validaNFe();
    }

    /**
     * @param $notafiscal
     * @return void
     */
    public function insereRelacnflotenfe(Notafiscal $notafiscal)
    {
        $relacnflotenfe = new Relacnflotenfe();
        $relacnflotenfe->setIdstatusrelnfe(Notafiscal::ID_STATUS_NFE_TRANSMITIDA);

        $relacnflotenfe->setNrrelnflotenfe(NovoCodigo::geraCodigo(
                $notafiscal->getNrorg(), 'RELACLOTENFE'));
        $relacnflotenfe->setCdemplotenfe($notafiscal->getCdempresa());
        $relacnflotenfe->setNrlotenfe(self::getNrlote());
        $relacnflotenfe->setCdfilial($notafiscal->getCdfilial());
        $relacnflotenfe->setNrorg($notafiscal->getNrorg());
        $relacnflotenfe->setNrlanctonf($notafiscal->getNrlanctonf());
        $relacnflotenfe->setNrnotafiscalele($notafiscal->getNrnotafisc());
        $relacnflotenfe->setIdambrelotenfe(self::getAmbiente());
        //        $relacnflotenfe->setNracessorelnfe($this->v200->getChaveAcesso());
        $relacnflotenfe->setNracessorelnfe($this->v300->getChaveAcesso());
        $relacnflotenfe->setCdfiliemit($notafiscal->getCdfiliemit());
        $relacnflotenfe->setCdinscesta($notafiscal->getCdinscesta());
        $relacnflotenfe->setCdinscmuni($notafiscal->getCdinscmuni());
        $relacnflotenfe->setCdserie($notafiscal->getCdserie());
        //        $relacnflotenfe->setCdserienfe($this->v200->corpoNF['CDSERIE']);
        $relacnflotenfe->setCdserienfe($this->v300->corpoNF['CDSERIE']);
        $relacnflotenfe->setDtenviolotenfe(DateUtil::getDataAtual());
        $relacnflotenfe->setCdfilirece($notafiscal->getCdfilirece());
        $relacnflotenfe->setVrnotafisc($notafiscal->getVrnotafisc());
        $relacnflotenfe->setDtemissao($notafiscal->getDtemissao());
        $relacnflotenfe->setIdentrsaidop($notafiscal->getIdentrsaidop());
        $relacnflotenfe->persist();
    }

    /**
     * Grava o arquivo xml no banco de dados (Tabela ARQXMLNFE)
     * @param $nrorg
     * @param $xml Strring
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $idtipoarqxml
     * @throws \Exception
     */
    private function gravaXml($nrorg, $xml, $cdfilial, $nrlanctonf, $idtipoarqxml)
    {
        try {
            $xml = (string)str_replace('<?xml version="1.0" encoding="UTF-8"?>', ' ', $xml);
            $xml = str_replace("\n", '', $xml);
            /** @var $arqxml Arqxmlnfe */
            $arqxml = Arqxmlnfe::findOneByNrorgAndCdfilialAndNrlanctonfAndIdtipoarqxml(
                $nrorg,
                $cdfilial,
                $nrlanctonf,
                $idtipoarqxml
            );

            if ($arqxml) {
                $arqxml->setDsarqxmlnfe(substr($xml, 0, 2001));


            } else {
                $arqxml = new Arqxmlnfe();
                $arqxml->setNrorg($nrorg);
                $arqxml->setCdfilial($cdfilial);
                $arqxml->setNrlanctonf($nrlanctonf);
                $arqxml->setIdtipoarqxml($idtipoarqxml);
                $arqxml->setDsarqxmlnfe($xml);
            }
            $arqxml->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }

    }

}