/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: NDPCAL_DO_PRATO_PLA.SQL

Descrio:
Retorna o NDPCal do Prato.

P_CDFILIAL   - Cdigo da Filial. Pode ser passado vazio para consultar
               o NDPCal da receita padro.
P_CDSERVFILI - Cdigo do Servico. Pode ser passado vazio para consultar
               o NDPCal da receita padro ou por filial.
P_CDFILAUXPRAT - Cdigo da filial do prato.
P_CDPRATO    - Cdigo do Prato.

Criao:
04/03/99 - Ederson

Manutenes:
05/03/99 - Zed     - Implementacao Inicial

****************************************************************
             CORREO DO CLCULO DO NDPCAL%
****************************************************************
A frmula usada para clculo do NDPCal% :

NDPCAL% = NPU * Sum(Protenas) * 4
          ------------------------
                    VCT

onde:

NPU = Somatrio das protenas dos componentes da receita multiplicados
      pelo Fator de Clculo de NDPCAL, informado na tela de parametrizao
      por grupo de produtos, sendo 0,7 para alimentos de origem animal;
      0,6 para leguminosas e 0,5 para cereais, frutas e outros vegetais.
Sum(Protenas) = Somatrio das protenas dos componentes da receita
VCT = "Valor Calrico Total" = Soma(carboidratos) x 4 +
                               Soma(lipdeos)    x 9  +
                               Soma(protenas)   x 4  +
                               Soma(lcoois)     x 7
****************************************************************
@C:\WIN_V1.53\SQL\ORACLE\7160_PLASTP_NDPCAL_DO_PRATO_PLA.SQL

*/

CREATE OR REPLACE
PROCEDURE NDPCAL_DO_PRATO_PLA
  (P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_CDFILIPROC   IN FILIAL.CDFILIAL%TYPE,
   P_CDSERVFILI   IN SERVICO.CDSERVFILI%TYPE,
   P_CDFILAUXPRAT IN PRATO.CDFILAUXPRAT%TYPE,
   P_CDPRATO      IN PRATO.CDPRATO%TYPE,
   P_CDFILCENTPME IN FILIAL.CDFILCENTPME%TYPE DEFAULT NULL,   
   P_CDTIPOCARD   IN TIPOCARD.CDTIPOCARD%TYPE DEFAULT NULL,
   P_MERENDA      IN VARCHAR2 DEFAULT 'N',
   P_NDPCAL_DO_PRATO OUT NUMBER) /*IDENTIFICADOR SE A PROCEDURE ESTA SENDO CHAMADA POR ROTINAS DO MERENDA*/
   IS

   V_NDPCAL    NUMBER;
   V_PROTLIQU  NUMBER; /* Sum (Protenas) */
   V_CALORIAS  NUMBER; /* VCT */
   V_NPU       NUMBER; /* Sum (Protena x Fator de Clculo) */

BEGIN

   /* ----------------- RETORNA O TOTAL DE CALORIAS DO PRATO ---------------- */

   FOR CUR_CALORIA IN
     (SELECT
           SUM(NP.QTNUTRPROD * QTPRODUTO) * 4 AS PROTEINA,
           SUM(NC.QTNUTRPROD * QTPRODUTO) * 4 AS CARBOIDRATO,
           SUM(NL.QTNUTRPROD * QTPRODUTO) * 9 AS LIPIDEO,
           SUM(NA.QTNUTRPROD * QTPRODUTO) * 7 AS ALCOOL
      FROM (
         /*RECEITA POR SERVIO*/
         SELECT R.CDPRODUTO, P.CDCARBOHIDRA, P.CDPROTEINA, P.CDLIPIDIOS, P.CDALCOOL,
                (10 * NVL(DECODE(PRODFILI.VRPESOUNIDFI, 0, R.VRPESOUNID, PRODFILI.VRPESOUNIDFI), R.VRPESOUNID) * R.QTCALCNUTR) AS QTPRODUTO
            FROM (SELECT MAX(CDCARBOHIDRA) CDCARBOHIDRA, MAX(CDPROTEINA) CDPROTEINA,
                         MAX(CDLIPIDIOS) CDLIPIDIOS, MAX(CDALCOOL) CDALCOOL FROM PARAMGERAL) P,
                 PRODFILI, PRATO P, 
                 TABLE (RECEITA_UTIL(P_CDFILIAL, P_CDSERVFILI, P.CDPRATO, P.CDFILAUXPRAT,'0', 'Z', P.CDPRATO,0,0,'N') ) R
           WHERE (P.CDPRATO = P_CDPRATO)
             AND (P.CDFILAUXPRAT = 'P' OR P.CDFILAUXPRAT = P_CDFILIAL)
             AND (R.CDPRATOPAI = P.CDPRATO) 
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND (PRODFILI.CDFILIAL  = P_CDFILIAL)
             AND (PRODFILI.CDPRODUTO = R.CDPRODUTO)
      ) R,
         NUTRPROD NC, NUTRPROD NP, NUTRPROD NL, NUTRPROD NA
      WHERE
         NP.CDPRODUTO   (+) = R.CDPRODUTO    AND
         NP.CDNUTRIENTE (+) = R.CDPROTEINA   AND
         NC.CDPRODUTO   (+) = R.CDPRODUTO    AND
         NC.CDNUTRIENTE (+) = R.CDCARBOHIDRA AND
         NL.CDPRODUTO   (+) = R.CDPRODUTO    AND
         NL.CDNUTRIENTE (+) = R.CDLIPIDIOS   AND
         NA.CDPRODUTO   (+) = R.CDPRODUTO    AND
         NA.CDNUTRIENTE (+) = R.CDALCOOL

      ) LOOP

      V_CALORIAS := NVL(CUR_CALORIA.CARBOIDRATO,0) + NVL(CUR_CALORIA.LIPIDEO,0) + NVL(CUR_CALORIA.PROTEINA,0) + NVL(CUR_CALORIA.ALCOOL,0);      
   END LOOP;

   /* ---------------- RETORNA O TOTAL DE PROTEINAS E O NPU DO PRATO ---------------- */

   FOR CUR_PROTEINA IN
     (SELECT SUM(PROTEINA) PROTEINA, SUM(NPU) NPU FROM (
      /*RECEITA POR SERVIO*/
      SELECT SUM(NP.QTNUTRPROD * 10 * NVL(DECODE(PRODFILI.VRPESOUNIDFI, 0, R.VRPESOUNID, PRODFILI.VRPESOUNIDFI), R.VRPESOUNID) * R.QTCALCNUTR) AS PROTEINA,
             SUM(NP.QTNUTRPROD * 10 * NVL(DECODE(PRODFILI.VRPESOUNIDFI, 0, R.VRPESOUNID, PRODFILI.VRPESOUNIDFI), R.VRPESOUNID) * R.QTCALCNUTR * NC.VRFATCALNDPC) AS NPU
        FROM PRODFILI, NUTRPROD NP, PRATO P, NDPCAL NC,
                 TABLE (RECEITA_UTIL(P_CDFILIAL, P_CDSERVFILI, P.CDPRATO, P.CDFILAUXPRAT,'0', 'Z', P.CDPRATO,0,0,'N') ) R
       WHERE (R.CDPRATOPAI = P.CDPRATO)
          AND (P.CDPRATO = P_CDPRATO)
          AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
          AND (R.CDPRODUTO       = NP.CDPRODUTO)
          AND (PRODFILI.CDFILIAL  = P_CDFILIAL)
          AND (PRODFILI.CDPRODUTO = R.CDPRODUTO)
          AND (NP.CDNUTRIENTE  = (SELECT MAX(CDPROTEINA) FROM PARAMGERAL))
          AND (NC.CDPRODUTO = (SELECT MAX(CDPRODUTO) FROM NDPCAL WHERE INSTR(R.CDPRODUTO,CDPRODUTO) = 1))
      )
      ) LOOP

      V_PROTLIQU := NVL(CUR_PROTEINA.PROTEINA,0);
      V_NPU      := NVL(CUR_PROTEINA.NPU,0);
   END LOOP;

   IF V_CALORIAS = 0 OR V_CALORIAS IS NULL THEN
      V_NDPCAL := 0;
   ELSE
      V_NDPCAL := NVL(V_PROTLIQU,0) * NVL(V_NPU,0) * 4 / V_CALORIAS;
   END IF;

   P_NDPCAL_DO_PRATO := ROUND(V_NDPCAL, 2);

END;
/

-- VERSAO --
BEGIN ATUVEROBJ('NDPCAL_DO_PRATO_PLA','0.00.000'); END;
/